import typing

if len(INSTANCE_CONFIG) == 3:
    ACCELERATOR, DEVICE, STATION = INSTANCE_CONFIG
    INSTANCE_NAME = None
elif len(INSTANCE_CONFIG) == 4:
    ACCELERATOR, DEVICE, STATION, INSTANCE_NAME = INSTANCE_CONFIG
else:
    raise RuntimeError("Invalid INSTANCE_CONFIG: " + str(INSTANCE_CONFIG))

def makeDoocsArray(values: typing.Iterable) -> str:
    return "      ".join(map(str, values))

def getNumberOfMotors() -> int:
    return len([x for x in MOTOR_AVAILABLE[STATION] if x])

def getFacilityPrefix() -> str:
    return 'XFEL' if STATION == 'XLAB1' else ACCELERATOR

def getMotorForStation(station: str) -> str:
    try:
        return MOTOR_DEVICE[station]
    except NameError:
        return 'MOTOR_DEVICE'
    except KeyError:
        return 'MOTOR_DEVICE'

RING_BUFFER = 50000
STORE_RATE = 100

SDM_REVISION = 'r3992'

feedbackIntegrationTicks = 10

#
# Settings which are filled by the accelerator-specific files
#

MOTOR_AVAILABLE = dict()

TOROID_LOCATION = dict()

MLO_DEVICE = dict()

MLO_LOCATION = dict()

LINK_LOCATION = dict()

LINK_LOCK_NUMBER = dict()

TIMER_LOCATION = dict()

MOTOR_CONVERSION_FACTOR = dict()

BCM_LOCATION = dict()

LONGITUDINAL_FEEDBACK_MONITORS = dict()

BBF_LOCATION = dict()

number_of_beamlines = 3
number_of_flattops = 3
number_of_regions = 16

if ACCELERATOR in ["XFEL", "LAB"]:
  toroid_compaction = 2
if ACCELERATOR in ["FLASH"]:
  toroid_compaction = 9

####
# Common hardware settings
####

BAM_DEVNO = 3
SDM_DEVNO = 4
MOTOR_DEVNO = 'devs8'
BPN = 15000

#
# Generic configuration
#
OPER_UID = -1
XPERT_UID = -1
XPERT_GID = 0
SVR_LOCATION = HOSTNAME.upper() + "._SVR"
FACILITY = ACCELERATOR + '.SDIAG'
if INSTANCE_NAME is None:
    WATCHDOG_ADDRESS = ACCELERATOR + '.SYSTEM/' + HOSTNAME.upper() + '.WATCH/SVR.BAM'
else:
    WATCHDOG_ADDRESS = ACCELERATOR + '.SYSTEM/' + HOSTNAME.upper() + f'.WATCH/SVR.BAM_{INSTANCE_NAME}'
MAIN_LOCATION = STATION
SWITCH_LOCATION = STATION + '.OPTICAL_SWITCH'
RPC_NUMBER = 610489844
CHARGE_SLOPE = {}
COMPRESSION_SLOPE = {}
DELAY_LINE_FOLDS = 4

# basic configuration that is different per accelerator
if ACCELERATOR in ["XFEL", "LAB"]:
    OPER_GID = 514  # xfelsdiag
    XPERT_GID = 514 # xfelsdiag

elif ACCELERATOR in ["FLASH"]:
    OPER_GID = 422   # flash
    XPERT_GID = 414  # ttfsdiag

#
# Other required output parameters for downstream scripting (e.g. postinst)
#
if INSTANCE_NAME is None:
    SERVERNAME = 'bam_server'
else:
    if ACCELERATOR == "FLASH":
        SERVERNAME = 'bam_{}_server'.format(INSTANCE_NAME)
    else:
        SERVERNAME = 'bam_{}_server'.format(INSTANCE_NAME.lower())

WORKDIR = '/export/doocs/server/' + SERVERNAME
EXECUTABLE_IN_PACKAGE = '/export/doocs/server/bam_server/bam_server'
MAKE_EXECUTABLE = 'init_scripts/bam-init.py'
USE_DOOCS_WATCHDOG=True
FILES_TO_SYMLINK_BETWEEN_INSTANCES = "init_scripts calibration alarm_send.sh"
SYSTEM_UID = "doocsadm"
SYSTEM_GID = "doocsadm"
